#ifndef _KNIGHTMARE_GAME
#define _KNIGHTMARE_GAME

#include "SDL_ttf.h"

#include "KMap.h"

#include "MusicManager.h"
#include "SFXManager.h"
#include "GLTManager.h"

class Knightmare
{
	friend class KnightmareApp;

    public:
        Knightmare(int level, int sfx_volume,
                   int music_volume, GLTManager *GLTM, SFXManager *SFXM);
        ~Knightmare();

        void clear(void);

        bool cycle(List<class VirtualController> *vcl, GLTManager *GLTM, SFXManager *SFXM, MusicManager *MusicM);
        void draw(GLTManager *GLTM);

        void set_music_volume(int v, MusicManager *MusicM);
        void set_sfx_volume(int v);
        void set_ambient_volume(int v, MusicManager *MusicM);

        int get_hiscore(void)
        {
            return m_hiscore;
        };
        void set_hiscore(int s)
        {
            m_hiscore = s;
        };

        class KObject *get_character(int player)
        {
                return m_characters[player];
        };

		bool game_paused();

		KMap *get_map(void) {
			return m_map;
		}
	
		int get_map_position(void) {
			return (int)get_map()->get_current_y();
		}

		void change_camera_angle(float a) {
			m_camera_angle += a;
			if (m_camera_angle<0) m_camera_angle=0;
			if (m_camera_angle>=90) m_camera_angle=90;
		}

		float get_camera_angle() {
			return m_camera_angle;
		}

		void change_zoom(float z) {
			m_current_zoom *= z;
			if (m_current_zoom<0.125) m_current_zoom=0.125;
			if (m_current_zoom>=8.0) m_current_zoom=8.0;
		}

		void set_zoom(float z) {
			m_current_zoom = z;
			if (m_current_zoom<0.125) m_current_zoom=0.125;
			if (m_current_zoom>=8.0) m_current_zoom=8.0;
		}

		float get_zoom() {
			return m_current_zoom;
		}

		void setPreserveAngle(bool preserve) {
			m_preserve_angle = preserve;
		}
	
		List<Symbol> *getObstacleClasses() {
			return &obstacle_classes;
		}
	
		TTF_Font *getInGameFont() {
			return m_font16;
		}

    protected:
	
		TTF_Font *m_font16;	// in game font (for text messages like bonus points, weapons, etc.)

    	float m_camera_angle;
        float m_current_zoom;
        bool m_preserve_angle;

        int m_cycle;
		List<KObject> m_characters;	// TODO: this is never use still!!!
		KMap *m_map;

		// This list contains the names of all the object types which are
		// obstacles (columns, rivers, etc.)
		List<Symbol> obstacle_classes;
	
		int m_hiscore;

        // volumes:
        int m_sfx_volume, m_music_volume;

        // pause/map variables:
        int m_gamepaused;
        int m_gamepaused_timmer;
};

#endif

